import { Component, NgZone, OnInit } from '@angular/core';
import { ShoutboxDataService } from './services/ShoutboxData.service';
import { makeTestnetWallet } from './util/TestnetUtils';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit {
  lastMessageHash = ""
  contractAddress = "";
  initializing = true
  waiting = false
  title = 'httXrp Shoutbox';
  shouts: any[] = []
  sending = false
  newShout = {
    title: "",
    body: "",
    from: "",
    hash: "",
    date: "",
  }
  userWallet = {
    address: "",
    secret: ""
  }
  drainWallet = {
    address: "",
    secret: ""
  }

  constructor(
    private dataService: ShoutboxDataService,
    private zone: NgZone
  ) {
    this.contractAddress = dataService.getContractAddress()
  }

  submitShout = () => {
    this.sending = true
    this.dataService.submitShout(this.newShout)
      .then((hash) => {
        this.newShout = {
          title: "",
          body: "",
          from: "",
          hash: "",
          date: "",
        }
        this.lastMessageHash = hash
      })
      .finally(() => {
        this.sending = false
        this.waiting = true
        setTimeout(() => { this.waiting = false }, 20000)
      })
  }

  ngOnInit() {
    Promise.all([makeTestnetWallet(), makeTestnetWallet()])
      .then(async ([userWallet, drainWallet]) => {
        this.userWallet = userWallet
        this.drainWallet = drainWallet

        await this.dataService.initialize((shout: any) => {
          this.zone.run(_ => {
            this.shouts.unshift(shout)
            console.log(shout)
          })
        }, userWallet, drainWallet)
        this.initializing = false
      })
  }
}
