import { AMOUNT_DECIMALS, AMOUNT_FORMAT, NON_ZERO_TX_HASH } from "./protocol.constants"
import { Amount, ArgumentType, TxHash } from "./types"

export class DataParser {
    public static parse(type: ArgumentType | string[], value: any){
        if(typeof type === 'object' && type[0] != null){ //enum value type
            if(! (type as string[]).includes(value)){
                throw new Error(`Invalid enum value type: ${value} is not included in ${type}`)
            }
            return value
        }else if(typeof type === 'string'){ //known defined type
            if(!this.typeMap[type]){
                throw new Error('Unknown parameter type'+type)
            }
            return this.typeMap[type](value)
        }else{
            throw new Error('FRAMEWORK_ERR: Datachecker.check(..): unknown argument type '+String(type))
        }
    }

    private static typeMap : {[key in ArgumentType] : Function} = {
        Amount: DataParser.parseAmount,
        Boolean: DataParser.parseBoolean,
        String: DataParser.parseString,
        TxHash: DataParser.parseTxHash,
        any: DataParser.parseAny
    }


    private static parseAmount(input: any): Amount{
        if(typeof input === 'string' && typeof input !== 'number')
            throw new Error('Input is not a number')

        if(typeof input === 'string')
            input = Number.parseFloat(input)

        if(! AMOUNT_FORMAT.test(''+input)){
            throw new Error('Input did not match the specification for `Amount`. The maximum number of decimals is '+AMOUNT_DECIMALS)
        }

        return input
    }


    private static parseString(input: any): string{
        if(typeof input !== 'string')
            throw new Error('Input is not a string')
        return input
    }

    private static parseBoolean(input: any): boolean{
        switch (typeof input) {
            case 'boolean': return input;
            case 'string': {
                if(input === 'true') return true;
                if(input === 'false') return false;
                break;
            }
        }
        throw new Error('Input is not a boolean')
    }
    
    private static parseAny(input: any): any{
        return input
    }

    private static parseTxHash(input: any): TxHash{
        if(typeof input !== 'string' || !NON_ZERO_TX_HASH.test(input)){
            throw new Error(`Input '${input}'is not a trasnaction hash`)
        }
        return input
    }
}